"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createError = void 0;
const fields_1 = require("./fields");
function createError(message, attributes = {}, ctor = Error) {
    var _a;
    message = String(message || attributes.message || 'Unknown error!');
    if (!(message.toLowerCase()).includes('error')) {
        if ((_a = ctor.name) === null || _a === void 0 ? void 0 : _a.endsWith('Error'))
            message = ctor.name + ': ' + message;
        else
            message = 'Error: ' + message;
    }
    let err = (new ctor(message));
    Object.keys(attributes).forEach(k => {
        const isErrorAttribute = fields_1.COMMON_ERROR_FIELDS_EXTENDED.has(k);
        const isAutogeneratedErrorAttribute = fields_1.QUASI_STANDARD_ERROR_FIELDS.has(k);
        if (k === 'details') {
            err.details = {
                ...err.details,
                ...attributes[k],
            };
        }
        else if (isAutogeneratedErrorAttribute) {
            // strange...
            // ignore, don't allow overriding auto-generated props
        }
        else if (isErrorAttribute) {
            // attach directly
            ;
            err[k] = attributes[k];
        }
        else {
            err.details = err.details || {};
            err.details[k] = attributes[k];
        }
    });
    err.framesToPop = (err.framesToPop || 0) + 1;
    return err;
}
exports.createError = createError;
//# sourceMappingURL=util--create.js.map