"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeError = void 0;
const fields_1 = require("./fields");
// Normalize any thrown object into a true, normal error.
// NOTE: will *always* recreate the error. TODO evaluate if possible to improve?
// Anything can be thrown: undefined, string, number...
// But that's obviously not a good practice.
// Even Error-like objects are sometime fancy!
// - seen: in browser, sometimes, an error-like, un-writable object is thrown
// - seen: frozen
// - seen: non-enumerable props
// So we want to ensure a true, safe, writable error object.
function normalizeError(err_like = {}) {
    var _a, _b;
    // Yes, we always re-create in case
    const p = Object.getPrototypeOf(err_like);
    // should we restrict to global standard constructors? TBD
    const constructor = ((_b = (_a = p === null || p === void 0 ? void 0 : p.constructor) === null || _a === void 0 ? void 0 : _a.name) === null || _b === void 0 ? void 0 : _b.endsWith('Error')) ? p.constructor : Error;
    // https://stackoverflow.com/questions/1606797/use-of-apply-with-new-operator-is-this-possible
    const true_err = new (Function.prototype.bind.call(constructor, null, err_like.message || `(non-error caught: "${err_like}")`));
    // properly attach fields if they exist
    fields_1.COMMON_ERROR_FIELDS_EXTENDED.forEach(prop => {
        if (err_like[prop])
            true_err[prop] = err_like[prop];
    });
    return true_err;
}
exports.normalizeError = normalizeError;
//# sourceMappingURL=util--normalize.js.map