import {
	XError,
	XXError
} from './types'

import {
	QUASI_STANDARD_ERROR_FIELDS,
	COMMON_ERROR_FIELDS_EXTENDED,
} from './fields'

interface Attributes extends XXError {
	[k: string]: any
}

export function createError(message: string, attributes: Partial<Attributes> = {}, ctor = Error): XXError {
	message = String(message || attributes.message || 'Unknown error!')
	if (!(message.toLowerCase()).includes('error')) {
		if (ctor.name?.endsWith('Error'))
			message = ctor.name + ': ' + message
		else
			message = 'Error: ' + message
	}

	let err: XXError = (new ctor(message)) as any

	Object.keys(attributes).forEach(k => {
		const isErrorAttribute = COMMON_ERROR_FIELDS_EXTENDED.has(k as keyof XXError)
		const isAutogeneratedErrorAttribute = QUASI_STANDARD_ERROR_FIELDS.has(k as keyof XError)
		if (k === 'details') {
			err.details = {
				...err.details,
				...attributes[k],
			}
		}
		else if (isAutogeneratedErrorAttribute) {
			// strange...
			// ignore, don't allow overriding auto-generated props
		}
		else if (isErrorAttribute) {
			// attach directly
			;(err as any)[k] = attributes[k]
		}
		else {
			err.details = err.details || {}
			err.details[k] = attributes[k]
		}
	})
	err.framesToPop = (err.framesToPop || 0) + 1

	return err
}
